@@ -13,7 +13,8 @@ The code in this file is mainly lifted out of cmd.py in Python 2.2, with minor
 changes. Licensing should therefore be under the standard Python terms.  For
 details on the PSF (Python Software Foundation) standard license, see:
 
-http://www.python.org/2.2.3/license.html"""
+https://docs.python.org/2/license.html
+"""
 
 #*****************************************************************************
 #
@@ -24,49 +25,48 @@ http://www.python.org/2.2.3/license.html"""
 #
 #
 #*****************************************************************************
-from __future__ import print_function
 
 import bdb
 import functools
+import inspect
 import linecache
 import sys
+import warnings
+import re
 
 from IPython import get_ipython
-from IPython.utils import PyColorize, ulinecache
-from IPython.utils import coloransi, io, py3compat
+from IPython.utils import PyColorize
+from IPython.utils import coloransi, py3compat
 from IPython.core.excolors import exception_colors
 from IPython.testing.skipdoctest import skip_doctest
 
-# See if we can use pydb.
-has_pydb = False
-prompt = 'ipdb> '
-#We have to check this directly from sys.argv, config struct not yet available
-if '--pydb' in sys.argv:
-    try:
-        import pydb
-        if hasattr(pydb.pydb, "runl") and pydb.version>'1.17':
-            # Version 1.17 is broken, and that's what ships with Ubuntu Edgy, so we
-            # better protect against it.
-            has_pydb = True
-    except ImportError:
-        print("Pydb (http://bashdb.sourceforge.net/pydb/) does not seem to be available")
 
-if has_pydb:
-    from pydb import Pdb as OldPdb
-    #print "Using pydb for %run -d and post-mortem" #dbg
-    prompt = 'ipydb> '
-else:
-    from pdb import Pdb as OldPdb
+prompt = 'ipdb> '
+
+#We have to check this directly from sys.argv, config struct not yet available
+from pdb import Pdb as OldPdb
 
 # Allow the set_trace code to operate outside of an ipython instance, even if
 # it does so with some limitations.  The rest of this support is implemented in
 # the Tracer constructor.
+
+def make_arrow(pad):
+    """generate the leading arrow in front of traceback or debugger"""
+    if pad >= 2:
+        return '-'*(pad-2) + '> '
+    elif pad == 1:
+        return '>'
+    return ''
+
+
 def BdbQuit_excepthook(et, ev, tb, excepthook=None):
     """Exception hook which handles `BdbQuit` exceptions.
 
     All other exceptions are processed using the `excepthook`
     parameter.
     """
+    warnings.warn("`BdbQuit_excepthook` is deprecated since version 5.1",
+                  DeprecationWarning, stacklevel=2)
     if et==bdb.BdbQuit:
         print('Exiting Debugger.')
     elif excepthook is not None:
@@ -75,12 +75,19 @@ def BdbQuit_excepthook(et, ev, tb, excepthook=None):
         # Backwards compatibility. Raise deprecation warning?
         BdbQuit_excepthook.excepthook_ori(et,ev,tb)
 
+
 def BdbQuit_IPython_excepthook(self,et,ev,tb,tb_offset=None):
+    warnings.warn(
+        "`BdbQuit_IPython_excepthook` is deprecated since version 5.1",
+        DeprecationWarning, stacklevel=2)
     print('Exiting Debugger.')
 
 
 class Tracer(object):
-    """Class for local debugging, similar to pdb.set_trace.
+    """
+    DEPRECATED
+
+    Class for local debugging, similar to pdb.set_trace.
 
     Instances of this class, when called, behave like pdb.set_trace, but
     providing IPython's enhanced capabilities.
@@ -93,8 +100,11 @@ class Tracer(object):
     """
 
     @skip_doctest
-    def __init__(self,colors=None):
-        """Create a local debugger instance.
+    def __init__(self, colors=None):
+        """
+        DEPRECATED
+
+        Create a local debugger instance.
 
         Parameters
         ----------
@@ -112,13 +122,16 @@ class Tracer(object):
             from IPython.core.debugger import Tracer; debug_here = Tracer()
 
         Later in your code::
-        
+
             debug_here()  # -> will open up the debugger at that point.
 
         Once the debugger activates, you can use all of its regular commands to
         step through code, set breakpoints, etc.  See the pdb documentation
         from the Python standard library for usage details.
         """
+        warnings.warn("`Tracer` is deprecated since version 5.1, directly use "
+                      "`IPython.core.debugger.Pdb.set_trace()`",
+                      DeprecationWarning, stacklevel=2)
 
         ip = get_ipython()
         if ip is None:
@@ -126,12 +139,6 @@ class Tracer(object):
             sys.excepthook = functools.partial(BdbQuit_excepthook,
                                                excepthook=sys.excepthook)
             def_colors = 'NoColor'
-            try:
-                # Limited tab completion support
-                import readline
-                readline.parse_and_bind('tab: complete')
-            except ImportError:
-                pass
         else:
             # In ipython, we use its custom exception handler mechanism
             def_colors = ip.colors
@@ -169,6 +176,13 @@ class Tracer(object):
         self.debugger.set_trace(sys._getframe().f_back)
 
 
+RGX_EXTRA_INDENT = re.compile(r'(?<=\n)\s+')
+
+
+def strip_indentation(multiline_string):
+    return RGX_EXTRA_INDENT.sub('', multiline_string)
+
+
 def decorate_fn_with_doc(new_fn, old_fn, additional_text=""):
     """Make new_fn have old_fn's doc string. This is particularly useful
     for the ``do_...`` commands that hook into the help system.
@@ -177,70 +191,63 @@ def decorate_fn_with_doc(new_fn, old_fn, additional_text=""):
     def wrapper(*args, **kw):
         return new_fn(*args, **kw)
     if old_fn.__doc__:
-        wrapper.__doc__ = old_fn.__doc__ + additional_text
+        wrapper.__doc__ = strip_indentation(old_fn.__doc__) + additional_text
     return wrapper
 
 
-def _file_lines(fname):
-    """Return the contents of a named file as a list of lines.
-
-    This function never raises an IOError exception: if the file can't be
-    read, it simply returns an empty list."""
-
-    try:
-        outfile = open(fname)
-    except IOError:
-        return []
-    else:
-        out = outfile.readlines()
-        outfile.close()
-        return out
-
-
 class Pdb(OldPdb):
-    """Modified Pdb class, does not load readline."""
+    """Modified Pdb class, does not load readline.
 
-    def __init__(self,color_scheme='NoColor',completekey=None,
-                 stdin=None, stdout=None):
+    for a standalone version that uses prompt_toolkit, see
+    `IPython.terminal.debugger.TerminalPdb` and
+    `IPython.terminal.debugger.set_trace()`
+    """
+
+    def __init__(self, color_scheme=None, completekey=None,
+                 stdin=None, stdout=None, context=5, **kwargs):
+        """Create a new IPython debugger.
+        
+        :param color_scheme: Deprecated, do not use.
+        :param completekey: Passed to pdb.Pdb.
+        :param stdin: Passed to pdb.Pdb.
+        :param stdout: Passed to pdb.Pdb.
+        :param context: Number of lines of source code context to show when
+            displaying stacktrace information.
+        :param kwargs: Passed to pdb.Pdb.
+            The possibilities are python version dependent, see the python
+            docs for more info.
+        """
 
         # Parent constructor:
-        if has_pydb and completekey is None:
-            OldPdb.__init__(self,stdin=stdin,stdout=io.stdout)
-        else:
-            OldPdb.__init__(self,completekey,stdin,stdout)
+        try:
+            self.context = int(context)
+            if self.context <= 0:
+                raise ValueError("Context must be a positive integer")
+        except (TypeError, ValueError):
+                raise ValueError("Context must be a positive integer")
 
-        self.prompt = prompt # The default prompt is '(Pdb)'
+        # `kwargs` ensures full compatibility with stdlib's `pdb.Pdb`.
+        OldPdb.__init__(self, completekey, stdin, stdout, **kwargs)
 
         # IPython changes...
-        self.is_pydb = has_pydb
-
         self.shell = get_ipython()
 
         if self.shell is None:
+            save_main = sys.modules['__main__']
             # No IPython instance running, we must create one
             from IPython.terminal.interactiveshell import \
                 TerminalInteractiveShell
             self.shell = TerminalInteractiveShell.instance()
+            # needed by any code which calls __import__("__main__") after
+            # the debugger was entered. See also #9941.
+            sys.modules['__main__'] = save_main 
 
-        if self.is_pydb:
-
-            # interactiveshell.py's ipalias seems to want pdb's checkline
-            # which located in pydb.fn
-            import pydb.fns
-            self.checkline = lambda filename, lineno: \
-                             pydb.fns.checkline(self, filename, lineno)
-
-            self.curframe = None
-            self.do_restart = self.new_do_restart
-
-            self.old_all_completions = self.shell.Completer.all_completions
-            self.shell.Completer.all_completions=self.all_completions
-
-            self.do_list = decorate_fn_with_doc(self.list_command_pydb,
-                                                OldPdb.do_list)
-            self.do_l     = self.do_list
-            self.do_frame = decorate_fn_with_doc(self.new_do_frame,
-                                                 OldPdb.do_frame)
+        if color_scheme is not None:
+            warnings.warn(
+                "The `color_scheme` argument is deprecated since version 5.1",
+                DeprecationWarning, stacklevel=2)
+        else:
+            color_scheme = self.shell.colors
 
         self.aliases = {}
 
@@ -252,60 +259,59 @@ class Pdb(OldPdb):
         C = coloransi.TermColors
         cst = self.color_scheme_table
 
+        cst['NoColor'].colors.prompt = C.NoColor
         cst['NoColor'].colors.breakpoint_enabled = C.NoColor
         cst['NoColor'].colors.breakpoint_disabled = C.NoColor
 
+        cst['Linux'].colors.prompt = C.Green
         cst['Linux'].colors.breakpoint_enabled = C.LightRed
         cst['Linux'].colors.breakpoint_disabled = C.Red
 
+        cst['LightBG'].colors.prompt = C.Blue
         cst['LightBG'].colors.breakpoint_enabled = C.LightRed
         cst['LightBG'].colors.breakpoint_disabled = C.Red
 
-        self.set_colors(color_scheme)
+        cst['Neutral'].colors.prompt = C.Blue
+        cst['Neutral'].colors.breakpoint_enabled = C.LightRed
+        cst['Neutral'].colors.breakpoint_disabled = C.Red
+
 
         # Add a python parser so we can syntax highlight source while
         # debugging.
-        self.parser = PyColorize.Parser()
+        self.parser = PyColorize.Parser(style=color_scheme)
+        self.set_colors(color_scheme)
+
+        # Set the prompt - the default prompt is '(Pdb)'
+        self.prompt = prompt
 
     def set_colors(self, scheme):
         """Shorthand access to the color table scheme selector method."""
         self.color_scheme_table.set_active_scheme(scheme)
+        self.parser.style = scheme
 
     def interaction(self, frame, traceback):
-        self.shell.set_completer_frame(frame)
-        while True:
-            try:
-                OldPdb.interaction(self, frame, traceback)
-            except KeyboardInterrupt:
-                self.shell.write('\n' + self.shell.get_exception_only())
-                break
-            else:
-                break
+        try:
+            OldPdb.interaction(self, frame, traceback)
+        except KeyboardInterrupt:
+            self.stdout.write('\n' + self.shell.get_exception_only())
 
     def new_do_up(self, arg):
         OldPdb.do_up(self, arg)
-        self.shell.set_completer_frame(self.curframe)
     do_u = do_up = decorate_fn_with_doc(new_do_up, OldPdb.do_up)
 
     def new_do_down(self, arg):
         OldPdb.do_down(self, arg)
-        self.shell.set_completer_frame(self.curframe)
 
     do_d = do_down = decorate_fn_with_doc(new_do_down, OldPdb.do_down)
 
     def new_do_frame(self, arg):
         OldPdb.do_frame(self, arg)
-        self.shell.set_completer_frame(self.curframe)
 
     def new_do_quit(self, arg):
 
         if hasattr(self, 'old_all_completions'):
             self.shell.Completer.all_completions=self.old_all_completions
 
-        # Pdb sets readline delimiters, so set them back to our own
-        if self.shell.readline is not None:
-            self.shell.readline.set_completer_delims(self.shell.readline_delims)
-
         return OldPdb.do_quit(self, arg)
 
     do_q = do_quit = decorate_fn_with_doc(new_do_quit, OldPdb.do_quit)
@@ -316,20 +322,32 @@ class Pdb(OldPdb):
         self.msg("Restart doesn't make sense here. Using 'quit' instead.")
         return self.do_quit(arg)
 
-    def postloop(self):
-        self.shell.set_completer_frame(None)
-
-    def print_stack_trace(self):
+    def print_stack_trace(self, context=None):
+        if context is None:
+            context = self.context
+        try:
+            context=int(context)
+            if context <= 0:
+                raise ValueError("Context must be a positive integer")
+        except (TypeError, ValueError):
+                raise ValueError("Context must be a positive integer")
         try:
             for frame_lineno in self.stack:
-                self.print_stack_entry(frame_lineno, context = 5)
+                self.print_stack_entry(frame_lineno, context=context)
         except KeyboardInterrupt:
             pass
 
-    def print_stack_entry(self,frame_lineno,prompt_prefix='\n-> ',
-                          context = 3):
-        #frame, lineno = frame_lineno
-        print(self.format_stack_entry(frame_lineno, '', context), file=io.stdout)
+    def print_stack_entry(self, frame_lineno, prompt_prefix='\n-> ',
+                          context=None):
+        if context is None:
+            context = self.context
+        try:
+            context=int(context)
+            if context <= 0:
+                raise ValueError("Context must be a positive integer")
+        except (TypeError, ValueError):
+                raise ValueError("Context must be a positive integer")
+        print(self.format_stack_entry(frame_lineno, '', context), file=self.stdout)
 
         # vds: >>
         frame, lineno = frame_lineno
@@ -337,7 +355,15 @@ class Pdb(OldPdb):
         self.shell.hooks.synchronize_with_editor(filename, lineno, 0)
         # vds: <<
 
-    def format_stack_entry(self, frame_lineno, lprefix=': ', context = 3):
+    def format_stack_entry(self, frame_lineno, lprefix=': ', context=None):
+        if context is None:
+            context = self.context
+        try:
+            context=int(context)
+            if context <= 0:
+                print("Context must be a positive integer", file=self.stdout)
+        except (TypeError, ValueError):
+                print("Context must be a positive integer", file=self.stdout)
         try:
             import reprlib  # Py 3
         except ImportError:
@@ -388,7 +414,7 @@ class Pdb(OldPdb):
         ret.append(u'%s(%s)%s\n' % (link,lineno,call))
 
         start = lineno - 1 - context//2
-        lines = ulinecache.getlines(filename)
+        lines = linecache.getlines(filename)
         start = min(start, len(lines) - context)
         start = max(start, 0)
         lines = lines[start : start + context]
@@ -407,9 +433,9 @@ class Pdb(OldPdb):
         bp_mark = ""
         bp_mark_color = ""
 
-        scheme = self.color_scheme_table.active_scheme_name
-        new_line, err = self.parser.format2(line, 'str', scheme)
-        if not err: line = new_line
+        new_line, err = self.parser.format2(line, 'str')
+        if not err:
+            line = new_line
 
         bp = None
         if lineno in self.get_file_breaks(filename):
@@ -427,27 +453,12 @@ class Pdb(OldPdb):
         if arrow:
             # This is the line with the error
             pad = numbers_width - len(str(lineno)) - len(bp_mark)
-            if pad >= 3:
-                marker = '-'*(pad-3) + '-> '
-            elif pad == 2:
-                 marker = '> '
-            elif pad == 1:
-                 marker = '>'
-            else:
-                 marker = ''
-            num = '%s%s' % (marker, str(lineno))
-            line = tpl_line % (bp_mark_color + bp_mark, num, line)
+            num = '%s%s' % (make_arrow(pad), str(lineno))
         else:
             num = '%*s' % (numbers_width - len(bp_mark), str(lineno))
-            line = tpl_line % (bp_mark_color + bp_mark, num, line)
 
-        return line
+        return tpl_line % (bp_mark_color + bp_mark, num, line)
 
-    def list_command_pydb(self, arg):
-        """List command to use if we have a newer pydb installed"""
-        filename, first, last = OldPdb.parse_list_cmd(self, arg)
-        if filename is not None:
-            self.print_list_lines(filename, first, last)
 
     def print_list_lines(self, filename, first, last):
         """The printing (as opposed to the parsing part of a 'list'
@@ -462,7 +473,7 @@ class Pdb(OldPdb):
                 filename = self._exec_filename
 
             for lineno in range(first, last+1):
-                line = ulinecache.getline(filename, lineno)
+                line = linecache.getline(filename, lineno)
                 if not line:
                     break
 
@@ -474,12 +485,14 @@ class Pdb(OldPdb):
                 src.append(line)
                 self.lineno = lineno
 
-            print(''.join(src), file=io.stdout)
+            print(''.join(src), file=self.stdout)
 
         except KeyboardInterrupt:
             pass
 
     def do_list(self, arg):
+        """Print lines of code from the current stack frame
+        """
         self.lastcmd = 'list'
         last = None
         if arg:
@@ -495,7 +508,7 @@ class Pdb(OldPdb):
                 else:
                     first = max(1, int(x) - 5)
             except:
-                print('*** Error in argument:', repr(arg))
+                print('*** Error in argument:', repr(arg), file=self.stdout)
                 return
         elif self.lineno is None:
             first = max(1, self.curframe.f_lineno - 5)
@@ -513,6 +526,49 @@ class Pdb(OldPdb):
 
     do_l = do_list
 
+    def getsourcelines(self, obj):
+        lines, lineno = inspect.findsource(obj)
+        if inspect.isframe(obj) and obj.f_globals is obj.f_locals:
+            # must be a module frame: do not try to cut a block out of it
+            return lines, 1
+        elif inspect.ismodule(obj):
+            return lines, 1
+        return inspect.getblock(lines[lineno:]), lineno+1
+
+    def do_longlist(self, arg):
+        """Print lines of code from the current stack frame.
+
+        Shows more lines than 'list' does.
+        """
+        self.lastcmd = 'longlist'
+        try:
+            lines, lineno = self.getsourcelines(self.curframe)
+        except OSError as err:
+            self.error(err)
+            return
+        last = lineno + len(lines)
+        self.print_list_lines(self.curframe.f_code.co_filename, lineno, last)
+    do_ll = do_longlist
+
+    def do_debug(self, arg):
+        """debug code
+        Enter a recursive debugger that steps through the code
+        argument (which is an arbitrary expression or statement to be
+        executed in the current environment).
+        """
+        sys.settrace(None)
+        globals = self.curframe.f_globals
+        locals = self.curframe_locals
+        p = self.__class__(completekey=self.completekey,
+                           stdin=self.stdin, stdout=self.stdout)
+        p.use_rawinput = self.use_rawinput
+        p.prompt = "(%s) " % self.prompt.strip()
+        self.message("ENTERING RECURSIVE DEBUGGER")
+        sys.call_tracing(p.run, (arg, globals, locals))
+        self.message("LEAVING RECURSIVE DEBUGGER")
+        sys.settrace(self.trace_dispatch)
+        self.lastcmd = p.lastcmd
+
     def do_pdef(self, arg):
         """Print the call signature for any callable object.
 
@@ -560,35 +616,27 @@ class Pdb(OldPdb):
                       ('Globals', self.curframe.f_globals)]
         self.shell.find_line_magic('psource')(arg, namespaces=namespaces)
 
-    def checkline(self, filename, lineno):
-        """Check whether specified line seems to be executable.
+    def do_where(self, arg):
+        """w(here)
+        Print a stack trace, with the most recent frame at the bottom.
+        An arrow indicates the "current frame", which determines the
+        context of most commands. 'bt' is an alias for this command.
 
-        Return `lineno` if it is, 0 if not (e.g. a docstring, comment, blank
-        line or EOF). Warning: testing is not comprehensive.
-        """
-        #######################################################################
-        # XXX Hack!  Use python-2.5 compatible code for this call, because with
-        # all of our changes, we've drifted from the pdb api in 2.6.  For now,
-        # changing:
-        #
-        #line = linecache.getline(filename, lineno, self.curframe.f_globals)
-        # to:
-        #
-        line = linecache.getline(filename, lineno)
-        #
-        # does the trick.  But in reality, we need to fix this by reconciling
-        # our updates with the new Pdb APIs in Python 2.6.
-        #
-        # End hack.  The rest of this method is copied verbatim from 2.6 pdb.py
-        #######################################################################
+        Take a number as argument as an (optional) number of context line to
+        print"""
+        if arg:
+            context = int(arg)
+            self.print_stack_trace(context)
+        else:
+            self.print_stack_trace()
 
-        if not line:
-            print('End of file', file=self.stdout)
-            return 0
-        line = line.strip()
-        # Don't allow setting breakpoint at a blank line
-        if (not line or (line[0] == '#') or
-             (line[:3] == '"""') or line[:3] == "'''"):
-            print('*** Blank or comment', file=self.stdout)
-            return 0
-        return lineno
+    do_w = do_where
+
+
+def set_trace(frame=None):
+    """
+    Start debugging from `frame`.
+
+    If frame is not specified, debugging starts from caller's frame.
+    """
+    Pdb().set_trace(frame or sys._getframe().f_back)
